"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createPool = void 0;
const bindPool_1 = require("../binders/bindPool");
const Logger_1 = require("../Logger");
const createClientConfiguration_1 = require("./createClientConfiguration");
const createConnectionPool_1 = require("./createConnectionPool");
const createPoolConfiguration_1 = require("./createPoolConfiguration");
const pg_driver_1 = require("@slonik/pg-driver");
const node_events_1 = __importDefault(require("node:events"));
/**
 * @param connectionUri PostgreSQL [Connection URI](https://www.postgresql.org/docs/current/libpq-connect.html#LIBPQ-CONNSTRING).
 */
const createPool = async (connectionUri, clientConfigurationInput) => {
    const clientConfiguration = (0, createClientConfiguration_1.createClientConfiguration)(connectionUri, clientConfigurationInput);
    const createDriver = clientConfiguration.driverFactory ?? (0, pg_driver_1.createPgDriverFactory)();
    const driver = await createDriver({
        // TODO resolve name conflict between ClientConfiguration and DriverConfiguration
        driverConfiguration: clientConfiguration,
    });
    const events = new node_events_1.default();
    const pool = (0, createConnectionPool_1.createConnectionPool)({
        driver,
        events,
        ...(0, createPoolConfiguration_1.createPoolConfiguration)(clientConfiguration),
    });
    return (0, bindPool_1.bindPool)(events, Logger_1.Logger.child({
        poolId: pool.id(),
    }), pool, clientConfiguration);
};
exports.createPool = createPool;
//# sourceMappingURL=createPool.js.map