"use strict";
/* eslint-disable no-console */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const __1 = require("..");
// eslint-disable-next-line import/extensions
const termination_test_1 = require("./termination.test");
const pg_driver_1 = require("@slonik/pg-driver");
const ava_1 = __importDefault(require("ava"));
const node_child_process_1 = require("node:child_process");
const promises_1 = require("node:timers/promises");
(0, ava_1.default)('removes from pendingConnections when connection fails when there are multiple pending connections', async (t) => {
    t.timeout(10000);
    try {
        const output = (0, node_child_process_1.execSync)('docker --version', { encoding: 'utf8' });
        console.log('Docker CLI is available:', output.trim());
    }
    catch {
        console.log('Skipper the test. Docker CLI is not available.');
        return;
    }
    const { dsn, terminate } = await (0, termination_test_1.startTestContainer)();
    let connectionAttempt = 0;
    const pgDriverFactory = await (0, pg_driver_1.createPgDriverFactory)();
    const pool = await (0, __1.createPool)(dsn, {
        connectionRetryLimit: 0,
        connectionTimeout: 500,
        driverFactory: async (config) => {
            const driver = await pgDriverFactory(config);
            return {
                createClient: async () => {
                    connectionAttempt++;
                    console.error('Connection attempt:', connectionAttempt);
                    // Out of 3 connections, have the middle one fail and make sure it's removed properly from pendingConnections
                    if (connectionAttempt === 2) {
                        await (0, promises_1.setTimeout)(2000);
                        throw new Error('Connection failed.');
                    }
                    await (0, promises_1.setTimeout)(5000);
                    return driver.createClient();
                },
            };
        },
        idleTimeout: 30000,
        maximumPoolSize: 5,
    });
    await Promise.allSettled(Array.from({ length: 3 }).map(() => pool.connect(() => Promise.resolve())));
    t.like(pool.state(), {
        idleConnections: 2,
        pendingConnections: 0,
    });
    await pool.end();
    terminate();
});
//# sourceMappingURL=connectionFailure.test.js.map