"use strict";
/* eslint-disable no-console */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startTestContainer = void 0;
const __1 = require("..");
const ava_1 = __importDefault(require("ava"));
const get_port_1 = __importDefault(require("get-port"));
const node_child_process_1 = require("node:child_process");
const node_crypto_1 = require("node:crypto");
const promises_1 = require("node:timers/promises");
const startTestContainer = async () => {
    const dockerContainerName = `slonik-test-${(0, node_crypto_1.randomUUID)()}`;
    const servicePort = await (0, get_port_1.default)();
    const dockerArgs = [
        'run',
        '--name',
        dockerContainerName,
        '--rm',
        '-e',
        'POSTGRES_HOST_AUTH_METHOD=trust',
        '-p',
        servicePort + ':5432',
        // see packages/test-ssls/README.md
        'slonik-ssl-test',
        '-N 1000',
    ];
    const dockerProcess = (0, node_child_process_1.spawn)('docker', dockerArgs);
    dockerProcess.on('error', (error) => {
        console.error(error);
    });
    dockerProcess.stdout.on('data', (data) => {
        console.log(data.toString());
    });
    dockerProcess.stderr.on('data', (data) => {
        console.error(data.toString());
    });
    dockerProcess.on('exit', (code) => {
        console.log(`Docker process exited with code ${code}`);
    });
    await new Promise((resolve) => {
        dockerProcess.stdout.on('data', (data) => {
            if (data
                .toString()
                .includes('database system is ready to accept connections')) {
                resolve(undefined);
            }
        });
    });
    await (0, promises_1.setTimeout)(1000);
    const terminate = () => {
        (0, node_child_process_1.execSync)(`docker kill ${dockerContainerName}`);
    };
    return {
        servicePort,
        terminate,
    };
};
exports.startTestContainer = startTestContainer;
(0, ava_1.default)('makes a connection using SSL', async (t) => {
    const { servicePort, terminate } = await (0, exports.startTestContainer)();
    try {
        const searchParameters = new URLSearchParams();
        // TODO figure out how to test sslmode=require
        // We are now getting an error: SELF_SIGNED_CERT_IN_CHAIN
        searchParameters.set('sslmode', 'no-verify');
        searchParameters.set('sslrootcert', require.resolve('@slonik/test-ssls/root.crt'));
        searchParameters.set('sslcert', require.resolve('@slonik/test-ssls/slonik.crt'));
        searchParameters.set('sslkey', require.resolve('@slonik/test-ssls/slonik.key'));
        const pool = await (0, __1.createPool)(`postgresql://postgres@localhost:${servicePort}/postgres?${searchParameters}`);
        const result = await pool.one(__1.sql.unsafe `
      SELECT ssl
      FROM pg_stat_ssl
      JOIN pg_stat_activity
        ON pg_stat_ssl.pid = pg_stat_activity.pid;
    `);
        t.deepEqual(result, {
            ssl: true,
        });
        await pool.end();
    }
    finally {
        terminate();
    }
});
//# sourceMappingURL=ssl.test.js.map